/*
===============================================================================

  FILE:  SMwriter_sme.h
  
  CONTENTS:
  
    Writes a Streaming Mesh in a improved compressed format using a stream-order
    preserving connectivity coding scheme. 
  
  PROGRAMMERS:
  
    martin isenburg@cs.unc.edu
  
  COPYRIGHT:
  
    copyright (C) 2005  martin isenburg@cs.unc.edu
    
    This software is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  
  CHANGE HISTORY:
  
    09 January 2005 -- temporary renamed (smf->sme), but will become the new smc
    07 January 2005 -- improved cache hits ratio (sme->smf)
    05 January 2005 -- further simplified and improved in speed (smd->sme)
    27 July 2004 -- improved in speed and rates using fewer tables (smc->smd)
    3 January 2004 -- added support for pre-existing bounding box info
    28 October 2003 -- switched from Peter's hash to the STL hash_map
    15 September 2003 -- initial version created on the Monday after a tough
                         hiking weekend in Yosemite
  
===============================================================================
*/
#ifndef SMWRITER_SME_H
#define SMWRITER_SME_H

#include "smwriter.h"

#include <stdio.h>

class SMwriter_sme : public SMwriter
{
public:
  // additional mesh variables
  int nbits;

  // smwriter interface function implementations

  void add_comment(const char* comment);

  void set_nverts(int nverts);
  void set_nfaces(int nfaces);
  void set_boundingbox(const float* bb_min_f, const float* bb_max_f);

  void write_vertex(const float* v_pos_f);
  void write_triangle(const int* t_idx, const bool* t_final);
  void write_triangle(const int* t_idx);
  void write_finalized(int final_idx);

  void close();

  // smwriter_smc functions

  bool open(FILE* fd, int bits=16);

  SMwriter_sme();
  ~SMwriter_sme();

private:
  void write_header();
};

#endif
